@@ -8,22 +8,22 @@ import re
 import threading
 import time
 import urllib.parse
+import urllib.request
 from collections import defaultdict
+from collections.abc import Callable
+from collections.abc import Generator
+from collections.abc import Iterable
+from collections.abc import Mapping
+from collections.abc import MutableMapping
 from contextlib import contextmanager
 from contextlib import suppress
 from copy import copy
 from enum import Enum
+from http import HTTPStatus
+from re import Pattern
 from typing import TYPE_CHECKING
 from typing import Any
-from typing import Callable
 from typing import ClassVar
-from typing import Iterable
-from typing import Mapping
-from typing import MutableMapping
-from typing import Optional
-from typing import Pattern
-from typing import Tuple
-from typing import Union
 
 import werkzeug.http
 from werkzeug import Request
@@ -33,21 +33,27 @@ from werkzeug.datastructures import MultiDict
 from werkzeug.serving import make_server
 
 if TYPE_CHECKING:
+    import sys
     from ssl import SSLContext
+    from types import TracebackType
+
+    from werkzeug.serving import BaseWSGIServer
+
+    if sys.version_info >= (3, 11):
+        from typing import Self
+    else:
+        from typing_extensions import Self
 
 URI_DEFAULT = ""
 METHOD_ALL = "__ALL"
 
-HEADERS_T = Union[
-    Mapping[str, Union[str, Iterable[str]]],
-    Iterable[Tuple[str, str]],
-]
+HEADERS_T = Mapping[str, str | Iterable[str]] | Iterable[tuple[str, str]]
 
-HVMATCHER_T = Callable[[str, Optional[str], str], bool]
+HVMATCHER_T = Callable[[str, str | None, str], bool]
 
 
 class Undefined:
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "<UNDEFINED>"
 
 
@@ -91,7 +97,7 @@ class WaitingSettings:
         raise_assertions: bool = True,  # noqa: FBT001
         stop_on_nohandler: bool = True,  # noqa: FBT001
         timeout: float = 5,
-    ):
+    ) -> None:
         self.raise_assertions = raise_assertions
         self.stop_on_nohandler = stop_on_nohandler
         self.timeout = timeout
@@ -102,22 +108,24 @@ class Waiting:
 
     This class should not be instantiated directly."""
 
-    def __init__(self):
-        self._result = None
+    def __init__(self) -> None:
+        self._result: bool | None = None
         self._start = time.monotonic()
-        self._stop = None
+        self._stop: float | None = None
 
-    def complete(self, result: bool):  # noqa: FBT001
+    def complete(self, result: bool) -> None:  # noqa: FBT001
         self._result = result
         self._stop = time.monotonic()
 
     @property
     def result(self) -> bool:
-        return self._result
+        return bool(self._result)
 
     @property
     def elapsed_time(self) -> float:
         """Elapsed time in seconds"""
+        if self._stop is None:
+            raise TypeError("unsupported operand type(s) for -: 'NoneType' and 'float'")
         return self._stop - self._start
 
 
@@ -131,7 +139,7 @@ class HeaderValueMatcher:
 
     DEFAULT_MATCHERS: ClassVar[MutableMapping[str, Callable[[str | None, str], bool]]] = {}
 
-    def __init__(self, matchers: Mapping[str, Callable[[str | None, str], bool]] | None = None):
+    def __init__(self, matchers: Mapping[str, Callable[[str | None, str], bool]] | None = None) -> None:
         self.matchers = self.DEFAULT_MATCHERS if matchers is None else matchers
 
     @staticmethod
@@ -139,7 +147,7 @@ class HeaderValueMatcher:
         func = getattr(Authorization, "from_header", None)
         if func is None:  # Werkzeug < 2.3.0
             func = werkzeug.http.parse_authorization_header  # type: ignore[attr-defined]
-        return func(actual) == func(expected)
+        return func(actual) == func(expected)  # type: ignore
 
     @staticmethod
     def default_header_value_matcher(actual: str | None, expected: str) -> bool:
@@ -171,10 +179,10 @@ class QueryMatcher(abc.ABC):
 
     def match(self, request_query_string: bytes) -> bool:
         values = self.get_comparing_values(request_query_string)
-        return values[0] == values[1]
+        return bool(values[0] == values[1])
 
     @abc.abstractmethod
-    def get_comparing_values(self, request_query_string: bytes) -> tuple:
+    def get_comparing_values(self, request_query_string: bytes) -> tuple[Any, Any]:
         pass
 
 
@@ -183,7 +191,7 @@ class StringQueryMatcher(QueryMatcher):
     Matches a query for a string or bytes specified
     """
 
-    def __init__(self, query_string: bytes | str):
+    def __init__(self, query_string: bytes | str) -> None:
         """
         :param query_string: the query string will be compared to this string or bytes.
             If string is specified, it will be encoded by the encode() method.
@@ -195,13 +203,14 @@ class StringQueryMatcher(QueryMatcher):
 
         self.query_string = query_string
 
-    def get_comparing_values(self, request_query_string: bytes) -> tuple:
+    def get_comparing_values(self, request_query_string: bytes) -> tuple[bytes, bytes]:
+        if not isinstance(self.query_string, (str, bytes)):
+            raise TypeError("query_string must be a string, or a bytes-like object")
+
         if isinstance(self.query_string, str):
             query_string = self.query_string.encode()
-        elif isinstance(self.query_string, bytes):
-            query_string = self.query_string
         else:
-            raise TypeError("query_string must be a string, or a bytes-like object")
+            query_string = self.query_string
 
         return (request_query_string, query_string)
 
@@ -211,7 +220,7 @@ class MappingQueryMatcher(QueryMatcher):
     Matches a query string to a dictionary or MultiDict specified
     """
 
-    def __init__(self, query_dict: Mapping | MultiDict):
+    def __init__(self, query_dict: Mapping[str, str] | MultiDict[str, str]) -> None:
         """
         :param query_dict: if dictionary (Mapping) is specified, it will be used as a
             key-value mapping where both key and value should be string. If there are multiple
@@ -221,7 +230,7 @@ class MappingQueryMatcher(QueryMatcher):
         """
         self.query_dict = query_dict
 
-    def get_comparing_values(self, request_query_string: bytes) -> tuple:
+    def get_comparing_values(self, request_query_string: bytes) -> tuple[Mapping[str, str], Mapping[str, str]]:
         query = MultiDict(urllib.parse.parse_qsl(request_query_string.decode("utf-8")))
         if isinstance(self.query_dict, MultiDict):
             return (query, self.query_dict)
@@ -234,21 +243,21 @@ class BooleanQueryMatcher(QueryMatcher):
     Matches the query depending on the boolean value
     """
 
-    def __init__(self, result: bool):  # noqa: FBT001
+    def __init__(self, result: bool) -> None:  # noqa: FBT001
         """
         :param result: if this parameter is true, the query match will be always
             successful. Otherwise, no query match will be successful.
         """
         self.result = result
 
-    def get_comparing_values(self, request_query_string):  # noqa: ARG002
+    def get_comparing_values(self, request_query_string: bytes) -> tuple[bool, bool]:  # noqa: ARG002
         if self.result:
             return (True, True)
         else:
             return (True, False)
 
 
-def _create_query_matcher(query_string: None | QueryMatcher | str | bytes | Mapping) -> QueryMatcher:
+def _create_query_matcher(query_string: None | QueryMatcher | str | bytes | Mapping[str, str]) -> QueryMatcher:
     if isinstance(query_string, QueryMatcher):
         return query_string
 
@@ -312,10 +321,10 @@ class RequestMatcher:
         data: str | bytes | None = None,
         data_encoding: str = "utf-8",
         headers: Mapping[str, str] | None = None,
-        query_string: None | QueryMatcher | str | bytes | Mapping = None,
+        query_string: None | QueryMatcher | str | bytes | Mapping[str, str] = None,
         header_value_matcher: HVMATCHER_T | None = None,
         json: Any = UNDEFINED,
-    ):
+    ) -> None:
         if json is not UNDEFINED and data is not None:
             raise ValueError("data and json parameters are mutually exclusive")
 
@@ -340,7 +349,7 @@ class RequestMatcher:
         if header_value_matcher is not None:
             self.header_value_matcher = header_value_matcher
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """
         Returns the string representation of the object, with the known parameters.
         """
@@ -408,9 +417,9 @@ class RequestMatcher:
         except UnicodeDecodeError:
             return False
 
-        return json_received == self.json
+        return bool(json_received == self.json)
 
-    def difference(self, request: Request) -> list[tuple]:
+    def difference(self, request: Request) -> list[tuple[str, str, str | URIPattern]]:
         """
         Calculates the difference between the matcher and the request.
 
@@ -422,7 +431,7 @@ class RequestMatcher:
         matches the fields set in the matcher object.
         """
 
-        retval: list[tuple] = []
+        retval: list[tuple[str, Any, Any]] = []
 
         if not self.match_uri(request):
             retval.append(("uri", request.path, self.uri))
@@ -433,8 +442,8 @@ class RequestMatcher:
         if not self.query_matcher.match(request.query_string):
             retval.append(("query_string", request.query_string, self.query_string))
 
-        request_headers = {}
-        expected_headers = {}
+        request_headers: dict[str, str | None] = {}
+        expected_headers: dict[str, str] = {}
         for key, value in self.headers.items():
             if not self.header_value_matcher(key, request.headers.get(key), value):
                 request_headers[key] = request.headers.get(key)
@@ -467,11 +476,11 @@ class RequestHandlerBase(abc.ABC):
 
     def respond_with_json(
         self,
-        response_json,
-        status: int = 200,
+        response_json: Any,
+        status: int = HTTPStatus.OK.value,
         headers: Mapping[str, str] | None = None,
         content_type: str = "application/json",
-    ):
+    ) -> None:
         """
         Prepares a response with a serialized JSON object.
 
@@ -487,11 +496,11 @@ class RequestHandlerBase(abc.ABC):
     def respond_with_data(
         self,
         response_data: str | bytes = "",
-        status: int = 200,
+        status: int = HTTPStatus.OK.value,
         headers: HEADERS_T | None = None,
         mimetype: str | None = None,
         content_type: str | None = None,
-    ):
+    ) -> None:
         """
         Prepares a response with raw data.
 
@@ -508,7 +517,7 @@ class RequestHandlerBase(abc.ABC):
         self.respond_with_response(Response(response_data, status, headers, mimetype, content_type))
 
     @abc.abstractmethod
-    def respond_with_response(self, response: Response):
+    def respond_with_response(self, response: Response) -> None:
         """
         Prepares a response with the specified response object.
 
@@ -526,12 +535,12 @@ class RequestHandler(RequestHandlerBase):
     :param matcher: the matcher object
     """
 
-    def __init__(self, matcher: RequestMatcher):
+    def __init__(self, matcher: RequestMatcher) -> None:
         self.matcher = matcher
         self.request_handler: Callable[[Request], Response] | None = None
         self._hooks: list[Callable[[Request, Response], Response]] = []
 
-    def with_post_hook(self, hook: Callable[[Request, Response], Response]):
+    def with_post_hook(self, hook: Callable[[Request, Response], Response]) -> RequestHandler:
         self._hooks.append(hook)
         return self
 
@@ -557,7 +566,7 @@ class RequestHandler(RequestHandlerBase):
                 response = hook(request, response)
             return response
 
-    def respond_with_handler(self, func: Callable[[Request], Response]):
+    def respond_with_handler(self, func: Callable[[Request], Response]) -> None:
         """
         Registers the specified function as a responder.
 
@@ -565,7 +574,7 @@ class RequestHandler(RequestHandlerBase):
         """
         self.request_handler = func
 
-    def respond_with_response(self, response: Response):
+    def respond_with_response(self, response: Response) -> None:
         self.request_handler = lambda request: response
 
     def __repr__(self) -> str:
@@ -578,7 +587,7 @@ class RequestHandler(RequestHandlerBase):
         return retval
 
 
-class RequestHandlerList(list):
+class RequestHandlerList(list[RequestHandler]):
     """
     Represents a list of :py:class:`RequestHandler` objects.
 
@@ -631,26 +640,26 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         ssl_context: SSLContext | None = None,
         *,
         threaded: bool = False,
-    ):
+    ) -> None:
         """
         Initializes the instance.
 
         """
         self.host = host
         self.port = port
-        self.server = None
-        self.server_thread = None
-        self.assertions: list[str] = []
+        self.server: BaseWSGIServer | None = None
+        self.server_thread: threading.Thread | None = None
+        self.assertions: list[str | AssertionError] = []
         self.handler_errors: list[Exception] = []
         self.log: list[tuple[Request, Response]] = []
         self.ssl_context = ssl_context
         self.threaded = threaded
         self.no_handler_status_code = 500
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"<{self.__class__.__name__} host={self.host} port={self.port}>"
 
-    def clear(self):
+    def clear(self) -> None:
         """
         Clears and resets the state attributes of the object.
 
@@ -662,28 +671,28 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         self.clear_log()
         self.no_handler_status_code = 500
 
-    def clear_assertions(self):
+    def clear_assertions(self) -> None:
         """
         Clears the list of assertions
         """
 
         self.assertions = []
 
-    def clear_handler_errors(self):
+    def clear_handler_errors(self) -> None:
         """
         Clears the list of collected errors from handler invocations
         """
 
         self.handler_errors = []
 
-    def clear_log(self):
+    def clear_log(self) -> None:
         """
         Clears the list of log entries
         """
 
         self.log = []
 
-    def url_for(self, suffix: str):
+    def url_for(self, suffix: str) -> str:
         """
         Return an url for a given suffix.
 
@@ -712,7 +721,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
 
         return "{}://{}:{}{}".format(protocol, host, self.port, suffix)
 
-    def create_matcher(self, *args, **kwargs) -> RequestMatcher:
+    def create_matcher(self, *args: Any, **kwargs: Any) -> RequestMatcher:
         """
         Creates a :py:class:`.RequestMatcher` instance with the specified parameters.
 
@@ -721,13 +730,13 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
 
         return RequestMatcher(*args, **kwargs)
 
-    def thread_target(self):
+    def thread_target(self) -> None:
         """
         This method serves as a thread target when the server is started.
 
         This should not be called directly, but can be overridden to tailor it to your needs.
         """
-
+        assert self.server is not None
         self.server.serve_forever()
 
     def is_running(self) -> bool:
@@ -736,7 +745,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         """
         return bool(self.server)
 
-    def start(self):
+    def start(self) -> None:
         """
         Start the server in a thread.
 
@@ -755,14 +764,22 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         if self.is_running():
             raise HTTPServerError("Server is already running")
 
+        app = Request.application(self.application)
+
         self.server = make_server(
-            self.host, self.port, self.application, ssl_context=self.ssl_context, threaded=self.threaded
+            self.host,
+            self.port,
+            app,
+            ssl_context=self.ssl_context,
+            threaded=self.threaded,
         )
+
         self.port = self.server.port  # Update port (needed if `port` was set to 0)
-        self.server_thread = threading.Thread(target=self.thread_target)
+        # Explicitly make the new thread daemonic to avoid shutdown issues
+        self.server_thread = threading.Thread(target=self.thread_target, daemon=True)
         self.server_thread.start()
 
-    def stop(self):
+    def stop(self) -> None:
         """
         Stop the running server.
 
@@ -772,6 +789,8 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         Only a running server can be stopped. If the sever is not running, :py:class`HTTPServerError`
         will be raised.
         """
+        assert self.server is not None
+        assert self.server_thread is not None
         if not self.is_running():
             raise HTTPServerError("Server is not running")
         self.server.shutdown()
@@ -779,7 +798,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         self.server = None
         self.server_thread = None
 
-    def add_assertion(self, obj):
+    def add_assertion(self, obj: str | AssertionError) -> None:
         """
         Add a new assertion
 
@@ -790,7 +809,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         """
         self.assertions.append(obj)
 
-    def check(self):
+    def check(self) -> None:
         """
         Raises AssertionError or Errors raised in handlers.
 
@@ -799,7 +818,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         self.check_assertions()
         self.check_handler_errors()
 
-    def check_assertions(self):
+    def check_assertions(self) -> None:
         """
         Raise AssertionError when at least one assertion added
 
@@ -817,7 +836,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
 
             raise AssertionError(assertion)
 
-    def check_handler_errors(self):
+    def check_handler_errors(self) -> None:
         """
         Re-Raises any errors caused in request handlers
 
@@ -827,7 +846,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         if self.handler_errors:
             raise self.handler_errors.pop(0)
 
-    def respond_nohandler(self, request: Request, extra_message: str = ""):
+    def respond_nohandler(self, request: Request, extra_message: str = "") -> Response:
         """
         Add a 'no handler' assertion.
 
@@ -848,8 +867,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         :return: the response object what the handler responded, or a response which contains the error
         """
 
-    @Request.application  # type: ignore
-    def application(self, request: Request):
+    def application(self, request: Request) -> Response:
         """
         Entry point of werkzeug.
 
@@ -864,7 +882,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         self.log.append((request, response))
         return response
 
-    def __enter__(self):
+    def __enter__(self) -> Self:
         """
         Provide the context API
 
@@ -875,7 +893,12 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
             self.start()
         return self
 
-    def __exit__(self, *args, **kwargs):
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        traceback: TracebackType | None,
+    ) -> None:
         """
         Provide the context API
 
@@ -886,7 +909,7 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
             self.stop()
 
     @staticmethod
-    def format_host(host):
+    def format_host(host: str) -> str:
         """
         Formats a hostname so it can be used in a URL.
         Notably, this adds brackets around IPV6 addresses when
@@ -917,6 +940,9 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
 
     :param threaded: whether to handle concurrent requests in separate threads
 
+    :param startup_timeout: maximum time in seconds to wait for server readiness.
+        By default, no readiness check is performed.
+
     .. py:attribute:: no_handler_status_code
 
         Attribute containing the http status code (int) which will be the response
@@ -929,13 +955,14 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
 
     def __init__(
         self,
-        host=DEFAULT_LISTEN_HOST,
-        port=DEFAULT_LISTEN_PORT,
+        host: str = DEFAULT_LISTEN_HOST,
+        port: int = DEFAULT_LISTEN_PORT,
         ssl_context: SSLContext | None = None,
         default_waiting_settings: WaitingSettings | None = None,
         *,
         threaded: bool = False,
-    ):
+        startup_timeout: float | None = None,
+    ) -> None:
         """
         Initializes the instance.
         """
@@ -951,8 +978,34 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
             self.default_waiting_settings = WaitingSettings()
         self._waiting_settings = copy(self.default_waiting_settings)
         self._waiting_result: queue.LifoQueue[bool] = queue.LifoQueue(maxsize=1)
+        self.startup_timeout = startup_timeout
+        self._readiness_check_pending = False
 
-    def clear(self):
+    def start(self) -> None:
+        super().start()
+        self._readiness_check_pending = self.startup_timeout is not None
+        try:
+            self.wait_for_server_ready()
+        except Exception:
+            self.stop()
+            raise
+
+    def wait_for_server_ready(self) -> None:
+        """
+        Waits until the server is ready to serve requests.
+        """
+        if not self._readiness_check_pending:
+            return
+
+        url = self.url_for("/")
+        if not url.startswith(("http://", "https://")):
+            raise ValueError(f"Invalid URL generated for readiness check : {url}")  # noqa: EM102
+
+        with urllib.request.urlopen(url, timeout=self.startup_timeout) as resp:  # noqa: S310
+            if resp.status != HTTPStatus.OK.value or resp.read() != b"OK":
+                raise HTTPServerError("Readiness check failed with status code: {}".format(resp.status))
+
+    def clear(self) -> None:
         """
         Clears and resets the state attributes of the object.
 
@@ -963,7 +1016,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         self.clear_all_handlers()
         self.permanently_failed = False
 
-    def clear_all_handlers(self):
+    def clear_all_handlers(self) -> None:
         """
         Clears all types of the handlers (ordered, oneshot, permanent)
         """
@@ -995,7 +1048,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         data: str | bytes | None = None,
         data_encoding: str = "utf-8",
         headers: Mapping[str, str] | None = None,
-        query_string: None | QueryMatcher | str | bytes | Mapping = None,
+        query_string: None | QueryMatcher | str | bytes | Mapping[str, str] = None,
         header_value_matcher: HVMATCHER_T | None = None,
         handler_type: HandlerType = HandlerType.PERMANENT,
         json: Any = UNDEFINED,
@@ -1078,7 +1131,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         data: str | bytes | None = None,
         data_encoding: str = "utf-8",
         headers: Mapping[str, str] | None = None,
-        query_string: None | QueryMatcher | str | bytes | Mapping = None,
+        query_string: None | QueryMatcher | str | bytes | Mapping[str, str] = None,
         header_value_matcher: HVMATCHER_T | None = None,
         json: Any = UNDEFINED,
     ) -> RequestHandler:
@@ -1133,7 +1186,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         data: str | bytes | None = None,
         data_encoding: str = "utf-8",
         headers: Mapping[str, str] | None = None,
-        query_string: None | QueryMatcher | str | bytes | Mapping = None,
+        query_string: None | QueryMatcher | str | bytes | Mapping[str, str] = None,
         header_value_matcher: HVMATCHER_T | None = None,
         json: Any = UNDEFINED,
     ) -> RequestHandler:
@@ -1191,13 +1244,13 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         This method is primarily used when reporting errors.
         """
 
-        def format_handlers(handlers):
+        def format_handlers(handlers: list[RequestHandler]) -> list[str]:
             if handlers:
                 return ["    {!r}".format(handler.matcher) for handler in handlers]
             else:
                 return ["    none"]
 
-        lines = []
+        lines: list[str] = []
         lines.append("Ordered matchers:")
         lines.extend(format_handlers(self.ordered_handlers))
         lines.append("")
@@ -1209,7 +1262,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
 
         return "\n".join(lines)
 
-    def respond_nohandler(self, request: Request, extra_message: str = ""):
+    def respond_nohandler(self, request: Request, extra_message: str = "") -> Response:
         """
         Add a 'no handler' assertion.
 
@@ -1222,7 +1275,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
 
         return super().respond_nohandler(request, self.format_matchers() + extra_message)
 
-    def respond_permanent_failure(self):
+    def respond_permanent_failure(self) -> Response:
         """
         Add a 'permanent failure' assertion.
 
@@ -1251,6 +1304,10 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         :param request: the request object from the werkzeug library
         :return: the response object what the handler responded, or a response which contains the error
         """
+        if self._readiness_check_pending:
+            self._readiness_check_pending = False
+
+            return Response(HTTPStatus.OK.phrase, status=HTTPStatus.OK.value)
 
         if self.permanently_failed:
             return self.respond_permanent_failure()
@@ -1312,10 +1369,10 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
     @contextmanager
     def wait(
         self,
-        raise_assertions: bool | None = None,
-        stop_on_nohandler: bool | None = None,
+        raise_assertions: bool | None = None,  # noqa: FBT001
+        stop_on_nohandler: bool | None = None,  # noqa: FBT001
         timeout: float | None = None,
-    ):
+    ) -> Generator[Waiting, None, None]:
         """Context manager to wait until the first of following event occurs: all ordered and oneshot handlers were
         executed, unexpected request was received (if `stop_on_nohandler` is set to `True`), or time was out
 
@@ -1392,7 +1449,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         """
         return len(list(self.iter_matching_requests(matcher)))
 
-    def assert_request_made(self, matcher: RequestMatcher, *, count: int = 1):
+    def assert_request_made(self, matcher: RequestMatcher, *, count: int = 1) -> None:
         """
         Check the amount of log entries matching for the matcher specified. By
         default it verifies that exactly one request matching for the matcher
